import json
import logging
import os
import time
from dataclasses import dataclass
from typing import Dict, List, Optional, Set

from dotenv import load_dotenv
from langchain.tools import Tool

from core.ai import AIAssistant, AIConfig
from core.file_fetcher import FileFetcher
from core.git_manager import GitManager, GitConfig
from core.log_config import get_logger, setup_logging
from core.log_manager import LogManager

logger = get_logger(__name__)

@dataclass
class FileMemoryConfig:
    """配置文件记忆管理"""
    project_dir: str
    git_manager: GitManager
    ai_config: AIConfig
    # 可选的LogManager，用于获取上一轮修改信息
    log_manager: Optional[LogManager] = None


class FileDetail:
    """文件详细信息类"""
    pass


class FileMemory:
    """管理文件描述的记忆"""

    MEMORY_DIR = ".eng/memory"
    FILE_DETAILS_PATH = f"{MEMORY_DIR}/file_details.txt"
    GIT_ID_FILE = f"{MEMORY_DIR}/git_id"
    MAX_RETRIES = 3    # 最大重试次数
    RETRY_DELAY = 30    # 重试延迟（秒）
    # 每批次最大行数和字符数限制
    MAX_LINES_PER_BATCH = 10000  # 最大行数
    MAX_CHARS_PER_BATCH = 50000  # 最大字符数，约为 50KB
    MAX_FILES_PER_BATCH = 20  # 每批次最多处理的文件数

    def __init__(self, config: FileMemoryConfig):
        self.config = config
        self.memory_path = os.path.join(config.project_dir, self.FILE_DETAILS_PATH)
        self.git_id_path = os.path.join(config.project_dir, self.GIT_ID_FILE)

        # 保存LogManager引用
        self.log_manager = config.log_manager

        # 初始化 AI 助手
        self.ai_assistant = AIAssistant(config=self.config.ai_config, tools=[self._create_batch_description_tool()])

        # 初始化 Git 管理器
        self.git_manager = self.config.git_manager

        # 确保内存目录存在
        os.makedirs(os.path.dirname(self.memory_path), exist_ok=True)

    def _ensure_directories(self):
        """确保必要的目录存在"""
        memory_dir = os.path.join(self.config.project_dir, self.MEMORY_DIR)
        os.makedirs(memory_dir, exist_ok=True)

    def _get_failed_files_path(self) -> str:
        """获取失败文件记录的路径"""
        return os.path.join(self.config.project_dir, self.MEMORY_DIR, "failed_files.json")

    def _read_failed_files(self) -> List[str]:
        """读取处理失败的文件列表"""
        failed_files_path = self._get_failed_files_path()
        if os.path.exists(failed_files_path):
            try:
                with open(failed_files_path, 'r', encoding='utf-8') as f:
                    return json.load(f)
            except Exception as e:
                logger.error(f"读取失败文件列表出错: {str(e)}")
        return []

    def _write_failed_files(self, failed_files: List[str]) -> None:
        """写入处理失败的文件列表"""
        failed_files_path = self._get_failed_files_path()
        try:
            with open(failed_files_path, 'w', encoding='utf-8') as f:
                json.dump(failed_files, f, ensure_ascii=False, indent=2)
        except Exception as e:
            logger.error(f"写入失败文件列表出错: {str(e)}")

    def _create_batch_description_tool(self) -> Tool:
        """创建批量生成文件描述的工具"""
        from langchain.tools import Tool
        
        def process_file_descriptions(file_descriptions: str) -> Dict[str, str]:
            """
            处理文件描述列表
            
            Args:
                file_descriptions: JSON格式的文件描述列表，格式为 [{"fileName": "path/to/file.py", "desc": "文件描述"}]
                
            Returns:
                Dict[str, str]: 文件路径到描述的映射
            """
            try:
                descriptions = {}
                file_list = json.loads(file_descriptions)
                
                if not isinstance(file_list, list):
                    logger.error("错误：输入必须是一个列表")
                    return descriptions
                
                # 处理结果
                for item in file_list:
                    if isinstance(item, dict) and "fileName" in item and "desc" in item:
                        descriptions[item["fileName"]] = item["desc"]
                    else:
                        logger.warning(f"跳过无效的文件描述项: {item}")
                
                logger.info(f"成功处理了 {len(descriptions)} 个文件描述")
                return descriptions
            except json.JSONDecodeError:
                logger.error("错误：输入不是有效的 JSON 格式")
                return {}
            except Exception as e:
                logger.error(f"处理文件描述时出错: {str(e)}")
                return {}
        
        return Tool(
            name="process_file_descriptions",
            description="处理文件描述列表，输入必须是JSON格式的列表，每个元素包含fileName和desc字段",
            func=process_file_descriptions,
            return_direct=True
        )

    def _generate_batch_file_descriptions(self, files_with_content: List[Dict[str, str]]) -> Dict[str, str]:
        """
        批量生成文件描述
        
        Args:
            files_with_content: 包含文件路径和内容的列表，格式为 [{"filepath": "path/to/file.py", "content": "..."}]
            
        Returns:
            Dict[str, str]: 文件路径到描述的映射
        """
        # 构建提示词
        files_text = ""
        for i, file_info in enumerate(files_with_content):
            files_text += f"\n--- 文件 {i+1}: {file_info['filepath']} ---\n{file_info['content']}\n"
        
        prompt = f"""
请分析以下多个代码文件，并为每个文件生成一个简短的中文描述（每个不超过100字）。
描述应该包含：
1. 文件的主要功能
2. 包含的关键类或函数
3. 与其他文件的主要交互（如果明显的话）

{files_text}

请使用process_file_descriptions工具返回结果，输入必须是一个JSON格式的列表，每个元素包含fileName和desc字段。
例如：
[
  {{"fileName": "path/to/file1.py", "desc": "这个文件实现了..."}},
  {{"fileName": "path/to/file2.py", "desc": "这个文件定义了..."}}
]

请确保每个文件都有对应的描述，并且描述准确反映文件的功能和内容。
"""
        
        # 尝试生成描述，最多重试MAX_RETRIES次
        descriptions = {}
        failed_files = []
        file_paths = [file_info["filepath"] for file_info in files_with_content]
        
        for attempt in range(self.MAX_RETRIES):
            try:
                logger.info(f"尝试批量生成文件描述（第{attempt+1}次尝试）")
                
                # 使用工具生成描述
                descriptions = self.ai_assistant.generate_response(prompt, use_tools=True)
                
                # 如果返回的不是字典，可能是字符串响应
                if not isinstance(descriptions, dict):
                    logger.error(f"工具返回了非预期的结果类型: {type(descriptions)}")
                    descriptions = {}
                
                # 检查是否所有文件都有描述
                missing_files = [
                    file_path for file_path in file_paths
                    if file_path not in descriptions
                ]
                
                if missing_files:
                    failed_files.extend(missing_files)
                    logger.warning(f"以下文件未能生成描述: {missing_files}")
                
                # 如果有成功处理的文件，则返回结果
                if descriptions:
                    return descriptions
            
            except Exception as e:
                logger.error(f"批量生成文件描述失败（第{attempt+1}次尝试）: {str(e)}")
            
            # 如果不是最后一次尝试，则等待后重试
            if attempt < self.MAX_RETRIES - 1:
                logger.info(f"等待 {self.RETRY_DELAY} 秒后重试...")
                time.sleep(self.RETRY_DELAY)
        
        # 所有尝试都失败，记录失败的文件
        self._update_failed_files(file_paths)
        
        # 返回空结果
        return descriptions

    def _update_failed_files(self, new_failed_files: List[str]) -> None:
        """更新失败文件列表"""
        if not new_failed_files:
            return
            
        # 读取现有失败文件列表
        existing_failed_files = self._read_failed_files()
        
        # 合并并去重
        all_failed_files = list(set(existing_failed_files + new_failed_files))
        
        # 写入更新后的列表
        self._write_failed_files(all_failed_files)
        logger.info(f"更新了失败文件列表，共 {len(all_failed_files)} 个文件")

    def process_failed_files(self) -> Dict[str, str]:
        """处理之前失败的文件"""
        failed_files = self._read_failed_files()
        if not failed_files:
            logger.info("没有需要处理的失败文件")
            return {}
            
        logger.info(f"开始处理 {len(failed_files)} 个失败文件")
        
        # 准备文件内容
        files_with_content = []
        for filepath in failed_files:
            content = self._get_file_content(filepath)
            if content.strip():  # 跳过空文件
                files_with_content.append({"filepath": filepath, "content": content})
        
        # 按批次处理文件
        descriptions = self._process_files_in_batches(files_with_content)
        
        # 更新失败文件列表
        if descriptions:
            # 找出成功处理的文件
            processed_files = list(descriptions.keys())
            # 更新失败文件列表
            new_failed_files = [f for f in failed_files if f not in processed_files]
            self._write_failed_files(new_failed_files)
            
            logger.info(f"成功处理了 {len(processed_files)} 个之前失败的文件，还有 {len(new_failed_files)} 个文件失败, 如果存在要忽略的文件，请在项目根目录下配置 .eng/.engignore 配置方式同.gitignore")
        
        return descriptions


    def _get_file_content(self, filepath: str) -> str:
        """获取文件内容"""
        try:
            full_path = os.path.join(self.config.project_dir, filepath)
            with open(full_path, "r", encoding="utf-8") as f:
                return f.read()
        except Exception as e:
            logger.error(f"读取文件 {filepath} 失败: {str(e)}, 如果要忽略该文件，请在项目根目录下配置 .eng/.engignore 配置方式同.gitignore")
            return ""

    def _process_files_in_batches(self, files_with_content: List[Dict[str, str]]) -> Dict[str, str]:
        """将文件分批处理"""
        all_descriptions = {}
        current_batch = []
        current_lines = 0
        current_size = 0

        
        for file_info in files_with_content:
            content = file_info["content"]
            lines = len(content.splitlines())
            chars = len(content)
            
            if lines == 0:
                continue
                
            # 检查是否需要开始新批次
            # 如果当前批次已满或添加此文件会超出限制，则处理当前批次并开始新批次
            if (current_batch and (
                len(current_batch) >= self.MAX_FILES_PER_BATCH or
                current_lines + lines > self.MAX_LINES_PER_BATCH or
                current_size + chars > self.MAX_CHARS_PER_BATCH
            )):
                # 处理当前批次
                logger.info(f"处理批次: {len(current_batch)} 个文件，共 {current_lines} 行，{current_size} 字符")
                batch_descriptions = self._generate_batch_file_descriptions(current_batch)
                all_descriptions.update(batch_descriptions)
                
                # 重置批次
                current_batch = [file_info]
                current_lines = lines
                current_size = chars
            else:
                current_batch.append(file_info)
                current_lines += lines
                current_size += chars
        
        # 处理最后一个批次
        if current_batch:
            logger.info(f"处理最后一个批次: {len(current_batch)} 个文件，共 {current_lines} 行，{current_size} 字符")
            batch_descriptions = self._generate_batch_file_descriptions(current_batch)
            all_descriptions.update(batch_descriptions)
        
        return all_descriptions

    def _process_files_chunk(self, files: List[str]) -> Dict[str, str]:
        """处理一组文件，生成描述"""
        # 准备文件内容
        files_with_content = []
        for filepath in files:
            content = self._get_file_content(filepath)
            if content.strip():  # 跳过空文件
                files_with_content.append({"filepath": filepath, "content": content})
        
        # 按批次处理文件
        return self._process_files_in_batches(files_with_content)

    def _read_git_id(self) -> str:
        """读取保存的 Git ID"""
        if not os.path.exists(self.git_id_path):
            return ""
        with open(self.git_id_path, "r") as f:
            return f.read().strip()

    def _write_git_id(self, git_id: str) -> None:
        """写入当前 Git ID"""
        with open(self.git_id_path, "w") as f:
            f.write(git_id)

    def _read_file_details(self) -> Dict[str, str]:
        """读取文件描述信息"""
        if not os.path.exists(self.memory_path):
            return {}

        details = {}
        with open(self.memory_path, "r", encoding="utf-8") as f:
            for line in f:
                if ":" in line:
                    filename, description = line.strip().split(":", 1)
                    details[filename] = description
        return details

    def _write_file_details(self, details: Dict[str, str]) -> None:
        """写入文件描述信息"""
        with open(self.memory_path, "w", encoding="utf-8") as f:
            for filename, description in sorted(details.items()):
                f.write(f"{filename}:{description}\n")

    def update_file_details(self) -> None:
        """更新文件描述信息"""
        # 获取所有文件
        all_files = set(FileFetcher.get_all_files_without_ignore(self.config.project_dir))
        
        # 读取现有描述
        existing_details = self._read_file_details()

        
        # 如果有LogManager，使用它获取上一轮修改的文件
        if self.log_manager:
            # 获取上一轮修改的文件
            log_modified_files = self._get_last_round_modified_files()
            
            # 只处理LogManager中标记为修改的文件
            files_to_process = list(log_modified_files & all_files)
            
            # 删除不存在的文件的描述
            existing_details = {
                k: v for k, v in existing_details.items() if k in all_files
            }
            
            logger.info(f"使用LogManager方式更新文件描述，处理{len(files_to_process)}个修改的文件")
        else:
            # 如果没有LogManager，回退到Git方式
            current_git_id = self.git_manager.get_current_commit_id()
            saved_git_id = self._read_git_id()
            files_to_process = self._get_changed_files_git(all_files, existing_details, current_git_id, saved_git_id)
            logger.info(f"使用Git方式更新文件描述，处理{len(files_to_process)}个文件")

        # 处理需要更新的文件
        if files_to_process:
            new_descriptions = self._process_files_chunk(files_to_process)
            existing_details.update(new_descriptions)

        # 保存结果
        self._write_file_details(existing_details)
        if not self.log_manager:
            # 只有使用Git方式时才更新Git ID
            current_git_id = self.git_manager.get_current_commit_id()
            self._write_git_id(current_git_id)

    def _get_last_round_modified_files(self) -> set:
        """
        从LogManager获取上一轮修改的文件列表
        
        Returns:
            set: 上一轮修改的文件路径集合
        """
        if not self.log_manager:
            logger.info("未提供LogManager，无法获取上一轮修改的文件")
            return set()
        
        try:
            # 获取当前轮次
            current_round = self.log_manager.get_current_round()
            
            # 获取上一轮的日志条目
            if current_round > 1:
                prev_round = current_round - 1
                log_entry = self.log_manager.get_issue_round_log_entry(prev_round, include_diff=True)
                
                if log_entry and log_entry.modified_files:
                    # 从diff_info中提取文件路径
                    modified_files = set()
                    for diff_info in log_entry.modified_files:
                        if diff_info.file_name and diff_info.is_create:
                            modified_files.add(diff_info.file_name)
                    
                    logger.info(f"从LogManager获取到上一轮({prev_round})修改的文件: {len(modified_files)}个")
                    return modified_files
            return set()
        except Exception as e:
            logger.error(f"获取上一轮修改的文件失败: {str(e)}")
            return set()

    def _get_changed_files_git(self, all_files: Set[str], existing_details: Dict[str, str], 
                             current_git_id: str, saved_git_id: Optional[str]) -> List[str]:
        """使用Git方式获取需要处理的文件列表"""
        if saved_git_id:
            # 获取自上次运行以来修改的文件
            changed_files = set(
                self.git_manager.get_changed_files(saved_git_id, current_git_id)
            ) & all_files
            
            logger.info(f"从Git获取到变更文件: {len(changed_files)}个")
            
            new_files = all_files - set(existing_details.keys())
            logger.info(f"检测到新文件: {len(new_files)}个")
            
            return list(changed_files | new_files)
        else:
            # 首次运行，处理所有文件
            return list(all_files)

    @classmethod
    def get_file_descriptions(cls, project_dir: str) -> Dict[str, str]:
        """获取文件描述的静态方法"""
        memory_path = os.path.join(project_dir, cls.FILE_DETAILS_PATH)

        if not os.path.exists(memory_path):
            return {}

        try:
            descriptions = {}
            with open(memory_path, "r", encoding="utf-8") as f:
                for line in f:
                    if ":" in line:
                        filename, description = line.strip().split(":", 1)
                        descriptions[filename] = description
            return descriptions
        except Exception as e:
            logger.error(f"读取文件描述失败: {str(e)}")
            return {}

    @classmethod
    def get_selected_file_descriptions(cls, project_dir: str, files: List[str]) -> Dict[str, str]:
        """获取文件描述的静态方法"""
        memory_path = os.path.join(project_dir, cls.FILE_DETAILS_PATH)

        if not os.path.exists(memory_path):
            return {}
        try:
            descriptions = {}
            with open(memory_path, "r", encoding="utf-8") as f:
                for line in f:
                    if ":" in line:
                        filename, description = line.strip().split(":", 1)
                        if filename in files:
                            descriptions[filename] = description
            return descriptions
        except Exception as e:
            logger.error(f"读取文件描述失败: {str(e)}")
            return {}

if __name__ == "__main__":
    setup_logging(log_level=logging.DEBUG)
    load_dotenv()
    project_dir = "../."
    memory = FileMemory(
        FileMemoryConfig(
            project_dir=project_dir,
            ai_config=AIConfig(temperature=1, model_name="coder-model"),
            git_manager=GitManager(config=GitConfig(repo_path=project_dir)),
            log_manager=None
        )
    )

    memory.update_file_details()



import os
import shutil
import tempfile
import uuid
from dataclasses import dataclass
from typing import Optional

from core.ai import AIConfig
from core.chat_processor import ChatProcessor, ChatProcessorConfig
from core.code_engineer import CodeEngineer, CodeEngineerConfig
from core.decision import DecisionProcess
from core.diff import Diff
from core.file_memory import FileMemory, FileMemoryConfig
from core.file_selector import FileSelector
from core.git_manager import GitManager, GitConfig
from core.log_config import get_logger
from core.log_manager import LogManager, LogConfig
from core.prompt_generator import PromptGenerator, PromptData
from core.version_manager import VersionManager

logger = get_logger(__name__)

@dataclass
class WorkflowEngineConfig:
    project_dir: str
    issue_id:int
    core_model:str = "gpt-4o"
    data_model:str = "gpt-4o"
    core_template: float = 0.7
    data_template: float = 0.7
    max_retry: int = 3,
    default_branch: str = "main"
    mode: str = "client" # ["client", "bot"] bot模式下，每次进行工作时，会hard reset到issues的最新分支上
    base_url: Optional[str] = None
    api_key: Optional[str] = None
    github_remote_url: Optional[str] =None
    github_token: Optional[str] = None


class WorkflowEngine:
    CODE_TIMES = 0
    CHAT_TIMES = 0
    """
    工作流引擎，协调版本管理、日志管理和AI交互
    """
    def __init__(self, config: WorkflowEngineConfig):
        """
        初始化工作流引擎
        
        Args:
            config: 工作流配置
        """
        self.CODE_TIMES = 0
        self.CHAT_TIMES = 0
        # 存储原始配置
        self.original_config = config
        
        # 根据模式设置工作目录
        if config.mode == "bot":
            # 创建临时目录作为工作区
            self.temp_dir = os.path.join(
                tempfile.gettempdir(),
                f"bella-bot-{config.issue_id}-{str(uuid.uuid4())[:8]}"
            )
            os.makedirs(self.temp_dir, exist_ok=True)
            # 更新配置以使用临时目录
            self.config = WorkflowEngineConfig(
                project_dir=self.temp_dir,
                **{k: v for k, v in vars(config).items() if k != 'project_dir'}
            )
            logger.info(f"Bot模式：创建临时工作目录 {self.temp_dir}")
        else:
            # 客户端模式直接使用指定的目录
            self.config = config
            self.temp_dir = None
            logger.info("当前为client模式")

        self.project_dir = os.path.abspath(self.config.project_dir)
        # 创建AI配置
        self.core_ai_config = AIConfig(
            model_name=config.core_model,
            temperature=config.core_template,
            base_url=config.base_url,
            api_key=config.api_key
        )
        
        self.data_ai_config = AIConfig(
            model_name=config.data_model,
            temperature=config.data_template,
            base_url=config.base_url,
            api_key=config.api_key
        )
        
        # 创建Git配置
        self.git_config = GitConfig(
            repo_path=self.project_dir,
            remote_url=config.github_remote_url or os.getenv("GIT_REMOTE"),
            auth_token=config.github_token or os.getenv("GITHUB_TOKEN"),
            default_branch=config.default_branch
        )
        
        # 创建日志配置
        self.log_config = LogConfig(
            project_dir=self.project_dir,
            issue_id=config.issue_id,
            mode=config.mode
        )
        
        # 初始化管理器
        self.git_manager = GitManager(config=self.git_config)

        # 检出对应的分支
        if self.config.mode == "bot":
            try:
                # 重置到issue对应的分支
                self.git_manager.reset_to_issue_branch(self.config.issue_id)
                logger.info(f"成功初始化Bot模式环境，工作目录: {self.temp_dir}")
            except Exception as e:
                logger.error(f"初始化Bot模式环境失败: {str(e)}")
                self._cleanup_environment()
                raise

        self.log_manager = LogManager(config=self.log_config)
        
        # 初始化文件记忆管理，传入log_manager
        self.file_memory = FileMemory(
            config=FileMemoryConfig(
                git_manager=self.git_manager,
                ai_config=self.data_ai_config,
                project_dir=self.project_dir,
                log_manager=self.log_manager
            )
        )
        self.version_manager = VersionManager(
            issue_id=config.issue_id,
            ai_config=self.core_ai_config,
            log_manager=self.log_manager,
            git_manager=self.git_manager,
            file_memory=self.file_memory
        )
        self.file_selector = FileSelector(
            self.project_dir,
            self.config.issue_id,
            ai_config=self.core_ai_config
        )

        # 初始化代码工程师
        self.code_engineer_config = CodeEngineerConfig(
            project_dir=self.project_dir,
            ai_config=self.core_ai_config
        )
        self.engineer = CodeEngineer(
            self.code_engineer_config,
            self.log_manager,
            Diff(self.data_ai_config)
        )
        
        # 初始化聊天处理器
        self.chat_processor = ChatProcessor(
            ai_config=self.core_ai_config,
            log_manager=self.log_manager,
            config=ChatProcessorConfig(system_prompt="你是一个项目助手，负责回答关于代码库的问题。下面会给出用户的问题以及相关的项目文件信息。")
        )
        
        # 初始化决策环境
        self.decision_env = DecisionProcess(
            ai_config=self.core_ai_config,
            version_manager=self.version_manager
        )

    def _prepare_memory(self):
        current_round = self.log_manager.get_current_round()

        # 如果轮次大于1，增量更新上一轮修改的文件详细信息
        if self.file_memory and current_round > 1:
            self.file_memory.update_file_details()
            logger.info("已更新文件详细信息")
    
    def process_requirement(self, user_requirement: str) -> Optional[str]:
        """
        处理用户需求
        
        Args:
            user_requirement: 用户需求

        Returns:
            str: 处理结果的响应文本
        """
        try:

            response = self._process_requirement_internal(user_requirement)
            
            # 如果是bot模式，在结束时清理临时目录
            if self.config.mode == "bot":
                self._cleanup_environment()
            
            return response
        except Exception as e:
            logger.error(f"处理需求时发生错误: {str(e)}")
            raise


    def _finalize_changes(self, mode: str, comment_text: str) -> bool:
        """
        完成更改，在bot模式下提交并推送更改

        Args:
            mode: 工作模式 ("client" 或 "bot")
            comment_text: comment内容

        Returns:
            bool: 操作是否成功
        """
        if mode == "bot":
            self.git_manager.commit(f"Issues #{self.config.issue_id} - Changes by Bella-Issues-Bot")
            self.git_manager.push()
            self.git_manager.add_issue_comment(self.config.issue_id, comment_text)
        return True
        
    def _cleanup_environment(self) -> None:
        """
        清理工作环境，删除临时目录
        """
        if self.config.mode == "bot" and self.temp_dir and os.path.exists(self.temp_dir):
            try:
                # 关闭git仓库连接
                if hasattr(self, 'git_manager') and self.git_manager:
                    self.git_manager.delete_local_repository()
                
                # 删除临时目录
                shutil.rmtree(self.temp_dir, ignore_errors=True)
                logger.info(f"已清理临时工作目录: {self.temp_dir}")
            except Exception as e:
                logger.warning(f"清理临时目录时出错: {str(e)}")
                # 即使清理失败也不抛出异常，让主流程继续

    def _process_requirement_internal(self, user_requirement: str) -> Optional[str]:
        """
        内部处理需求的方法
        
        Args:
            user_requirement: 用户需求
            
        Returns:
            str: 处理结果
        """
        # 先通过决策环境分析需求类型
        decision_result = self.decision_env.analyze_requirement(user_requirement)
        
        logger.info(f"决策结果: 是否需要修改代码={decision_result.needs_code_modification}, "
                    f"理由={decision_result.reasoning}")
        
        if decision_result.needs_code_modification:
            # 执行代码修改流程
            response = self._run_code_generation_workflow(user_requirement)
        else: 
            # 执行对话流程
            response = self._run_chat_workflow(user_requirement)
        
        # 如果是Bot模式且有GitHub配置，自动回复到issue
        if self.config.mode == "bot":
            try:
                self._finalize_changes(mode=self.config.mode, comment_text=response)
                logger.info(f"更改已经推送到远端，并添加了Issue评论")
            except Exception as e:
                logger.error(f"添加Issue评论时出错: {str(e)}")
                
        return response
    
    def _run_code_generation_workflow(self, user_requirement: str) -> Optional[str]:
        """
        执行代码生成流程，基于example_code_generate.py的逻辑
        
        Args:
            user_requirement: 用户需求
            
        Returns:
            str: 处理结果
        """
        logger.info("开始执行代码生成流程")

        # 确定当前版本
        requirement, history = self.version_manager.ensure_version_and_generate_context(user_requirement)

        self._prepare_memory()

        # 生成提示词
        user_prompt = self._get_user_prompt(requirement, history)

        # 根据提示词修改代码
        success, response = self.engineer.process_prompt(prompt=user_prompt)

        # 提交更改
        if success:
            return response
        else:
            self.CODE_TIMES += 1
            if self.CODE_TIMES >= self.config.max_retry:
                logger.error("code workflow超过最大重试次数")
                return self._run_chat_workflow(user_requirement)
            else:
                return self._run_code_generation_workflow(user_requirement)
    
    def _run_chat_workflow(self, user_requirement: str) -> Optional[str]:
        """
        执行聊天流程，基于example_chat_process.py的逻辑
        
        Args:
            user_requirement: 用户需求
            
        Returns:
            str: 处理结果
        """
        logger.info("开始执行聊天回复流程")

        self._prepare_memory()

        history = self.version_manager.get_formatted_history()

        # 生成提示词
        user_prompt = self._get_user_prompt(user_requirement, history)
        
        # 处理聊天请求
        response = self.chat_processor.process_chat(user_prompt)

        if(response):
            return response
        else:
            self.CHAT_TIMES += 1
            if self.CHAT_TIMES >= self.config.max_retry:
                logger.error("chat workflow超过最大重试次数")
                return None
            else:
                return self._run_chat_workflow(user_requirement)

    def _get_user_prompt(self, requirement: str, history: str) -> str:
        # 选择文件
        files = self.file_selector.select_files_for_requirement(requirement)
        descriptions = FileMemory.get_selected_file_descriptions(self.project_dir, files)

        # 准备提示词数据
        data = PromptData(
            requirement=requirement,
            project_dir=self.project_dir,
            steps=history,
            files=files,
            file_desc=descriptions
        )

        # 生成提示词
        return PromptGenerator.generatePrompt(data)



"""
GitHub Workflow Generator

A module for generating GitHub Actions workflow files to integrate bella-issues-bot with GitHub.
Creates two workflows:
1. File Memory Initialization - Triggered on push to a configurable branch
2. Issue Processing Bot - Triggered when issues are created or commented on
"""

import argparse
import os
import sys
from pathlib import Path
from typing import Dict, Optional

from core.log_config import get_logger, setup_logging

logger = get_logger(__name__)

# Template for memory initialization workflow
MEMORY_INIT_TEMPLATE = """name: Initialize File Memory

on:
  workflow_dispatch:  # Allow manual triggering
    inputs:
      force_run:
        description: 'Force execution even for automated commits'
  push:
    branches:
      - {branch}

jobs:
  init-memory:
    runs-on: ubuntu-latest
    permissions:
      contents: write
    if: ${{{{ !contains(github.event.head_commit.message, '[skip memory]') || github.event_name == 'workflow_dispatch' }}}}
    steps:
      - name: Checkout code
        uses: actions/checkout@v3
        with:
          fetch-depth: 0

      - name: Set up Python
        uses: actions/setup-python@v4
        with:
          python-version: '3.10'

      - name: Install bella-issues-bot
        run: |
          python -m pip install --upgrade pip
          pip install bella-issues-bot{package_version}

      - name: Initialize file memory
        env:
          OPENAI_API_KEY: ${{{{ secrets.OPENAI_API_KEY }}}}
          OPENAI_API_BASE: ${{{{ secrets.OPENAI_API_BASE }}}}
          GIT_REMOTE: ${{{{ github.server_url }}}}/${{{{ github.repository }}}}
          GITHUB_TOKEN: ${{{{ secrets.GIT_TOKEN }}}}
        run: |
          bella-file-memory --mode bot -b {branch} -m {model} -t {temperature} --git-url "${{{{ github.server_url }}}}/${{{{ github.repository }}}}" --git-token "${{{{ secrets.GIT_TOKEN }}}}" -u "${{{{ secrets.OPENAI_API_BASE }}}}" -k "${{{{ secrets.OPENAI_API_KEY }}}}"
"""

# Template for issue processing workflow
ISSUE_PROCESS_TEMPLATE = """name: Process Issues with bella-issues-bot

on:
  issues:
    types: [opened]
  issue_comment:
    types: [created]

jobs:
  process-issue:
    runs-on: ubuntu-latest
    permissions:
      contents: write
      issues: write
    if: ${{{{ github.event_name == 'issues' || !startsWith(github.event.comment.body, 'bella-issues-bot已处理：') }}}}
    steps:
      - name: Checkout code
        uses: actions/checkout@v3
        with:
          fetch-depth: 0

      - name: Set up Python
        uses: actions/setup-python@v4
        with:
          python-version: '3.10'

      - name: Install bella-issues-bot
        run: |
          python -m pip install --upgrade pip
          pip install bella-issues-bot{package_version}

      - name: Extract issue info
        id: issue
        run: |
          if [[ "${{{{ github.event_name }}}}" == "issues" ]]; then
            echo "issue_id=${{{{ github.event.issue.number }}}}" >> $GITHUB_OUTPUT
            echo "requirement<<EOF" >> $GITHUB_OUTPUT
            echo "${{{{ github.event.issue.body }}}}" >> $GITHUB_OUTPUT
            echo "EOF" >> $GITHUB_OUTPUT
          else
            echo "issue_id=${{{{ github.event.issue.number }}}}" >> $GITHUB_OUTPUT
            echo "requirement<<EOF" >> $GITHUB_OUTPUT
            echo "${{{{ github.event.comment.body }}}}" >> $GITHUB_OUTPUT
            echo "EOF" >> $GITHUB_OUTPUT
          fi

      - name: Process issue with bella-issues-bot
        env:
          OPENAI_API_KEY: ${{{{ secrets.OPENAI_API_KEY }}}}
          OPENAI_API_BASE: ${{{{ secrets.OPENAI_API_BASE }}}}
          GIT_REMOTE: ${{{{ github.server_url }}}}/${{{{ github.repository }}}}
          GITHUB_TOKEN: ${{{{ secrets.GIT_TOKEN }}}}
          ISSUE_ID: ${{{{ steps.issue.outputs.issue_id }}}}
        run: |
          # Run bella-issues-bot in bot mode - it will handle branch creation and pushing
          bella-issues-bot --mode bot -b {base_branch} --issue-id ${{{{ steps.issue.outputs.issue_id }}}} --core-model {core_model} --data-model {data_model} --core-temperature {core_temperature} --data-temperature {data_temperature} --requirement "${{{{ steps.issue.outputs.requirement }}}}" --git-url "${{{{ github.server_url }}}}/${{{{ github.repository }}}}" --git-token "${{{{ secrets.GIT_TOKEN }}}}" -u "${{{{ secrets.OPENAI_API_BASE }}}}" -k "${{{{ secrets.OPENAI_API_KEY }}}}"
"""

def generate_workflow_files(
    output_dir: str,
    base_branch: str = "main",
    model: str = "gpt-4o",
    core_model: Optional[str] = None,
    data_model: Optional[str] = None,
    temperature: float = 0.7,
    core_temperature: Optional[float] = None,
    data_temperature: Optional[float] = None,
    package_version: str = ""
) -> Dict[str, str]:
    """
    Generate GitHub workflow YAML files.
    
    Args:
        output_dir: Directory to write workflow files
        base_branch: Base branch for pull requests
        model: Default model to use for all operations
        core_model: Model for core operations (if different from model)
        data_model: Model for data operations (if different from model)
        temperature: Default temperature setting for all models
        core_temperature: Temperature for core model (if different)
        data_temperature: Temperature for data model (if different)
        package_version: Specific version of package to install (e.g. "==0.1.1")
        
    Returns:
        Dictionary mapping file paths to their contents
    """
    workflows_dir = os.path.join(output_dir, ".github", "workflows")
    os.makedirs(workflows_dir, exist_ok=True)
    
    # Format version specification if provided
    if package_version and not package_version.startswith("=="):
        package_version = f"=={package_version}"
    
    # Use provided models or default to the general model
    actual_core_model = core_model or model
    actual_data_model = data_model or model
    
    # Use provided temperatures or default to the general temperature
    actual_core_temp = core_temperature if core_temperature is not None else temperature
    actual_data_temp = data_temperature if data_temperature is not None else temperature
    
    # Generate memory initialization workflow
    memory_workflow_path = os.path.join(workflows_dir, "memory_init.yml")
    memory_workflow_content = MEMORY_INIT_TEMPLATE.format(
        branch=base_branch,
        model=actual_data_model,
        temperature=actual_data_temp,
        package_version=package_version
    )
    
    # Generate issue processing workflow
    issue_workflow_path = os.path.join(workflows_dir, "issue_process.yml")
    issue_workflow_content = ISSUE_PROCESS_TEMPLATE.format(
        core_model=actual_core_model,
        data_model=actual_data_model,
        core_temperature=actual_core_temp,
        data_temperature=actual_data_temp,
        base_branch=base_branch,
        package_version=package_version
    )
    
    # Write the files
    with open(memory_workflow_path, 'w') as f:
        f.write(memory_workflow_content)
    
    with open(issue_workflow_path, 'w') as f:
        f.write(issue_workflow_content)
    
    logger.info(f"Generated workflow files in {workflows_dir}")
    
    return {
        memory_workflow_path: memory_workflow_content,
        issue_workflow_path: issue_workflow_content
    }

def main() -> None:
    """Command line interface for GitHub workflow generator."""
    parser = argparse.ArgumentParser(description="Generate GitHub Actions workflows for bella-issues-bot integration")
    parser.add_argument("--output", "-o", type=str, default=".", help="Output directory (default: current directory)")
    parser.add_argument("--base-branch", "-b", type=str, default="main", help="Base branch for pull requests (default: main)")
    parser.add_argument("--model", "-m", type=str, default="gpt-4o", help="Default model for all operations (default: gpt-4o)")
    parser.add_argument("--core-model", "--cm", type=str, help="Model for core operations (defaults to --model)")
    parser.add_argument("--data-model", "--dm", type=str, help="Model for data operations (defaults to --model)")
    parser.add_argument("--temperature", "-t", type=float, default=0.7, help="Default temperature for all models (default: 0.7)")
    parser.add_argument("--core-temperature", "--ct", type=float, help="Temperature for core model (defaults to --temperature)")
    parser.add_argument("--data-temperature", "--dt", type=float, help="Temperature for data model (defaults to --temperature)")
    parser.add_argument("--package-version", "-v", type=str, default="", help="Specific package version to install (e.g. '0.1.1')")
    parser.add_argument("--log-level", "-l", type=str, choices=["DEBUG", "INFO", "WARNING", "ERROR"], default="INFO", help="Logging level")
    
    args = parser.parse_args()
    
    # Setup logging
    import logging
    setup_logging(log_level=getattr(logging, args.log_level))
    
    # Generate workflow files
    try:
        generate_workflow_files(
            output_dir=args.output,
            base_branch=args.base_branch,
            model=args.model,
            core_model=args.core_model,
            data_model=args.data_model,
            temperature=args.temperature,
            core_temperature=args.core_temperature,
            data_temperature=args.data_temperature,
            package_version=args.package_version
        )
        logger.info("Successfully generated GitHub workflow files")
    except Exception as e:
        logger.error(f"Error generating workflow files: {str(e)}")
        sys.exit(1)

if __name__ == "__main__":
    main()



import os
from typing import List, Optional, Set

from dotenv import load_dotenv
from langchain.tools import Tool
from pydantic import Field, BaseModel

from core.ai import AIAssistant, AIConfig
from core.file_fetcher import FileFetcher
from core.file_memory import FileMemory
from core.log_config import get_logger
from langchain_core.tools import StructuredTool

logger = get_logger(__name__)

class FileSelector:
    """
    使用 AI 辅助选择实现特定功能所需的文件
    """

    def __init__(self, project_dir: str, issues_id: int, ai_config: Optional[AIConfig] = None):
        """
        初始化 FileSelector
        
        Args:
            project_dir: 项目根目录
            ai_config: AI 配置，如果为 None 则使用默认配置
        """
        self.project_dir = project_dir
        self.issues_id = issues_id
        self.ai_config = ai_config or AIConfig()
        
        # 创建 AI 助手和工具
        self.select_files_tool = self._create_select_files_tool()
        self.ai_assistant = AIAssistant(
            config=self.ai_config,
            tools=[self.select_files_tool]
        )

    class _RequirementAnalyzerSchema(BaseModel):
        file_list: List[str] = Field(..., description="被选择的文件列表")

    def _create_select_files_tool(self) -> StructuredTool:
        """创建选择文件的工具"""
        return StructuredTool.from_function(
            name="select_files",
            description="选择实现功能所需的文件。输入文件相对路径列表进行选择，相对路径基于项目的根目录。",
            func=lambda **kwargs: kwargs["file_list"],
            args_schema=self._RequirementAnalyzerSchema,
            return_direct=True,
        )
    
    def select_files_for_requirement(self, requirement: str) -> List[str]:
        """
        根据需求选择所需的文件
        
        Args:
            requirement: 功能需求描述
            
        Returns:
            选择的文件列表
        """
        try:
            # 获取项目中的所有文件
            all_files = FileFetcher.get_all_files_without_ignore(self.project_dir)
            logger.info(f"获取到项目中的文件数量: {len(all_files)}")
            
            # 构建提示词
            prompt = self._build_prompt(requirement, all_files)
            
            # 使用 AI 助手生成响应，使用工具但不指定 tool_choice
            return self.ai_assistant.generate_response(
                prompt, 
                use_tools=True
            )
        except Exception as e:
            logger.error(f"select_files_for_requirement 工具执行异常: {str(e)}")
            return []
    
    def _build_prompt(self, requirement: str, all_files: Set[str]) -> str:
        """
        构建提示词
        
        Args:
            requirement: 功能需求
            all_files: 所有可用文件
            
        Returns:
            构建的提示词
        """
        file_str = "\n".join(all_files)
        files_memory = f"""
##角色：
你是一名资深的程序员，现在用户提出了一个需求，首先你要阅读项目代码和文档来了解项目名，你需要根据需求，决定阅读哪些文件。请你根据以下信息做出判断。
##以下是项目中的所有文件:
{file_str}
"""
        # 获取文件描述
        try:
            file_descriptions = FileMemory.get_file_descriptions(self.project_dir)
        except Exception as e:
            logger.warning("file memory is not exists")
            file_descriptions = None

        if file_descriptions:
            # 构建文件描述字符串
            file_str = "\n".join([
                f"- {file}：{file_descriptions.get(file, '无描述')}"
                for file in sorted(all_files)
            ])
            files_memory = f"""
##以下是项目中的所有文件及其功能描述：

{file_str}
"""

        
        return f"""
##需求：

{requirement}


{files_memory}

##请分析这个功能需求，并根据文件名和文件的描述，确定实现这个功能所需阅读的文件。
你的回答应该只包含文件名列表，每个文件名应该与上面列表中的完全匹配。
请使用 select_files 工具提交你的选择，工具需要的参数是一个list。

##原则：
1、如果你认为该文件和需求的关联性大，有助于你理解如何实现功能那么你应该选择
2、如果你认为该文件可能要在该功能的实现时被修改，那么需要选择
3、如果你认为需要分析项目的依赖配置或新增依赖，那么请选择配置文件
"""

if __name__ == "__main__":
    load_dotenv()
    requirement = """
    """
    project_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "../."))
    selector = FileSelector(project_dir, 5, ai_config=AIConfig(
        temperature=0.7,
        model_name="claude-3.5-sonnet"
    ))

    print(selector.select_files_for_requirement(requirement))


"""
聊天处理模块，负责非代码修改类的用户交互。

该模块提供以下功能:
1. 处理用户的问题、咨询和澄清请求
2. 使用上下文信息提供有针对性的回答
3. 记录聊天互动日志
"""

from copy import copy
from dataclasses import dataclass
from typing import Optional

from core.ai import AIAssistant, AIConfig
from core.log_config import get_logger
from core.log_manager import LogManager

logger = get_logger(__name__)

@dataclass
class ChatProcessorConfig:
    """聊天处理器配置"""
    system_prompt: str = "你是一个专业的开发助手，负责回答用户关于项目的问题和提供技术支持。下面会给出用户需求相关的代码和文档，以及历史迭代信息。"


class ChatProcessor:
    """
    聊天处理器，处理非代码修改类需求
    """

    def __init__(self, 
                 ai_config: AIConfig, 
                 log_manager: LogManager,
                 config: Optional[ChatProcessorConfig] = None):
        """
        初始化聊天处理器
        
        Args:
            ai_config: AI配置信息
            log_manager: 日志管理器
            config: 聊天处理器配置
        """
        self.config = config or ChatProcessorConfig()
        self.log_manager = log_manager
        
        # 设置系统提示词
        self.ai_config = copy(ai_config)
        self.ai_config.sys_prompt = self.config.system_prompt
        self.ai_assistant = AIAssistant(config=self.ai_config)

    def process_chat(self, user_query: str) -> str:
        """
        处理用户的聊天请求
        
        Args:
            user_query: 用户的问题或请求
            
        Returns:
            str: AI助手的回答
        """
        
        # 生成回答
        response = self.ai_assistant.generate_response(user_query)
        
        # 记录日志
        self.log_manager.archive_logs(
            sys_prompt=self.config.system_prompt,
            prompt=user_query,
            response=response
        )
        
        return response



import logging
import os
import sys
from logging.handlers import RotatingFileHandler

import colorlog

project_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "../."))
def setup_logging(log_dir=f'{project_dir}/logs', log_level=logging.INFO, log_file='app.log'):
    """
    设置日志配置，使用 colorlog 实现彩色控制台输出
    
    Args:
        log_dir: 日志文件目录
        log_level: 日志级别
        log_file: 日志文件名
    
    Returns:
        logging.Logger: 配置好的根日志记录器
    """
    # 确保日志目录存在
    os.makedirs(log_dir, exist_ok=True)
    
    # 创建根日志记录器
    root_logger = logging.getLogger()
    root_logger.setLevel(log_level)
    
    # 清除现有处理器
    for handler in root_logger.handlers[:]:
        root_logger.removeHandler(handler)
    
    # 创建彩色控制台处理器
    console_handler = colorlog.StreamHandler(stream=sys.stdout)
    console_handler.setLevel(log_level)
    console_format = colorlog.ColoredFormatter(
        '%(log_color)s%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        log_colors={
            'DEBUG': 'cyan',
            'INFO': 'green',
            'WARNING': 'yellow',
            'ERROR': 'red',
            'CRITICAL': 'red,bg_white',
        }
    )
    console_handler.setFormatter(console_format)
    
    # 创建文件处理器（带日志轮换）
    log_file_path = os.path.join(log_dir, log_file)
    file_handler = RotatingFileHandler(
        log_file_path, maxBytes=10*1024*1024, backupCount=5, encoding='utf-8'
    )
    file_handler.setLevel(log_level)
    file_format = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S'
    )
    file_handler.setFormatter(file_format)
    
    # 添加处理器到根日志记录器
    root_logger.addHandler(console_handler)
    root_logger.addHandler(file_handler)
    
    # 返回根日志记录器
    return root_logger

def get_logger(name, log_level=None):
    """
    获取指定名称的日志记录器
    
    Args:
        name: 日志记录器名称
        log_level: 日志级别，如果提供则覆盖默认级别
    
    Returns:
        logging.Logger: 指定名称的日志记录器
    """
    logger = logging.getLogger(name)
    if log_level is not None:
        logger.setLevel(log_level)
    return logger

# 如果直接运行此模块，则设置日志并进行测试
if __name__ == "__main__":
    setup_logging(log_level=logging.DEBUG)
    logger = get_logger(__name__)
    
    logger.debug("这是一条调试日志")
    logger.info("这是一条信息日志")
    logger.warning("这是一条警告日志")
    logger.error("这是一条错误日志")
    logger.critical("这是一条严重错误日志")
    
    print("\n日志配置测试完成，请检查控制台输出和logs目录下的日志文件。")


"""
File Memory Client

A standalone client for initializing and managing FileMemory using only GitManager.
This module provides both CLI and programmatic interfaces for updating file descriptions.
"""

import argparse
import logging
import os
import shutil
import sys
import tempfile
import uuid
from typing import Dict, Optional

from dotenv import load_dotenv

from core.ai import AIConfig
from core.file_memory import FileMemory, FileMemoryConfig
from core.git_manager import GitManager, GitConfig
from core.log_config import setup_logging, get_logger

logger = get_logger(__name__)


def initialize_file_memory(
    project_dir: str,
    base_branch: str = "main",
    model_name: str = "gpt-4o",
    temperature: float = 0.7,
    api_key: Optional[str] = None,
    base_url: Optional[str] = None,
    remote_url: Optional[str] = None,
    auth_token: Optional[str] = None,
) -> FileMemory:
    """
    Initialize FileMemory using GitManager without LogManager.
    
    Args:
        project_dir: Path to the project directory
        model_name: AI model to use for generating file descriptions
        temperature: Temperature setting for AI responses
        api_key: API key for AI service (will use env var if None)
        base_url: Base URL for AI service (will use default if None)
        remote_url: Git remote URL (will use env var if None)
        auth_token: Git authentication token (will use env var if None)
        
    Returns:
        Initialized FileMemory instance
    """
    # Create AI config
    ai_config = AIConfig(
        model_name=model_name,
        temperature=temperature,
        api_key=api_key,
        base_url=base_url
    )
    
    # Create Git config
    git_config = GitConfig(
        repo_path=project_dir,
        default_branch=base_branch,
        remote_url=remote_url or os.getenv("GIT_REMOTE_URL"),
        auth_token=auth_token or os.getenv("GITHUB_TOKEN")
    )
    
    # Initialize Git manager
    git_manager = GitManager(config=git_config)
    
    # Initialize and return FileMemory
    file_memory_config = FileMemoryConfig(
        project_dir=project_dir,
        git_manager=git_manager,
        ai_config=ai_config,
        log_manager=None  # Explicitly set to None as per requirements
    )
    
    return FileMemory(config=file_memory_config)


def update_file_descriptions(file_memory: FileMemory) -> None:
    """
    Update file descriptions using the given FileMemory instance.
    
    Args:
        file_memory: Initialized FileMemory instance
        
    Returns:
        Dictionary mapping file paths to their descriptions
    """
    return file_memory.update_file_details()


def process_failed_files(file_memory: FileMemory) -> Dict[str, str]:
    """
    Process previously failed files to generate their descriptions.
    
    Args:
        file_memory: Initialized FileMemory instance
        
    Returns:
        Dictionary mapping file paths to their descriptions
    """
    return file_memory.process_failed_files()


def main() -> None:
    """Command line interface for FileMemory client."""
    # Load environment variables
    load_dotenv()
    
    # Parse command line arguments
    parser = argparse.ArgumentParser(description="FileMemory Client - Update file descriptions for a project")
    parser.add_argument("--base-branch", "-b", type=str, default="main", help="Base branch for pull requests (default: main)")
    parser.add_argument("-d", "--directory", default=".", help="Project directory path (default: current directory)")
    parser.add_argument("-m", "--model", default="gpt-4o", help="AI model name (default: gpt-4o)")
    parser.add_argument("-t", "--temperature", type=float, default=0.7, help="AI temperature (default: 0.7)")
    parser.add_argument("-k", "--api-key", help="OpenAI API key (defaults to OPENAI_API_KEY env var)")
    parser.add_argument("-u", "--base-url", help="Base URL for API calls (optional)")
    parser.add_argument("--git-url", help="Git remote URL (defaults to GIT_REMOTE_URL env var)")
    parser.add_argument("--git-token", help="Git auth token (defaults to GITHUB_TOKEN env var)")
    parser.add_argument("-l", "--log-level", choices=["DEBUG", "INFO", "WARNING", "ERROR"], default="INFO", help="Logging level")
    parser.add_argument("--failed-only", action="store_true", help="Process only previously failed files")
    parser.add_argument("-md", "--mode", default="client", help="Project directory path (default: current directory)")
    args = parser.parse_args()
    
    # Setup logging
    setup_logging(log_level=getattr(logging, args.log_level))

    if args.mode == "bot":
        project_dir = os.path.join(
            tempfile.gettempdir(),
            f"bella-bot-memory-init-{str(uuid.uuid4())[:8]}"
        )
    else:
        # Get absolute path for project directory
        project_dir = os.path.abspath(args.directory)
        if not os.path.isdir(project_dir):
            logger.error(f"Project directory does not exist: {project_dir}")
            sys.exit(1)
    
    try:
        # Initialize FileMemory
        file_memory = initialize_file_memory(
            project_dir=project_dir,
            base_branch=args.base_branch,
            model_name=args.model,
            temperature=args.temperature,
            api_key=args.api_key,
            base_url=args.base_url,
            remote_url=args.git_url,
            auth_token=args.git_token
        )


        # Update file details or process failed files
        if args.failed_only:
            process_failed_files(file_memory)
            logger.info("Processed failed files")
        else:
            update_file_descriptions(file_memory)
            logger.info("Updated descriptions files")
        if args.mode == "bot":
            file_memory.git_manager.commit("Update file memory [skip memory]", add_all=False, files=[".eng/memory/file_details.txt", ".eng/memory/git_id"])
            file_memory.git_manager.pull()
            file_memory.git_manager.push()
            file_memory.git_manager.delete_local_repository()
    finally:
        if args.mode == "bot":
            # 删除临时目录
            shutil.rmtree(project_dir, ignore_errors=True)
            logger.info(f"已清理临时工作目录: {project_dir}")


if __name__ == "__main__":
    setup_logging(logging.DEBUG)
    main()



"""
决策环境模块，用于分析用户需求类型并确定处理流程。

该模块主要功能：
1. 分析用户输入的需求是否需要修改代码
2. 根据分析结果决定使用代码修改流程还是对话流程
3. 为AI助手提供决策工具
"""

from dataclasses import dataclass
from typing import Optional

from langchain_core.tools import StructuredTool
from pydantic import BaseModel, Field

from core.ai import AIAssistant, AIConfig
from core.log_config import get_logger
from core.version_manager import VersionManager

logger = get_logger(__name__)

@dataclass
class DecisionResult:
    """存储决策结果信息"""
    needs_code_modification: bool
    reasoning: str


class DecisionProcess:
    """
    决策环境类，用于确定用户需求是代码修改还是对话
    """
    
    def __init__(self, ai_config: AIConfig, version_manager: VersionManager):
        """
        初始化决策环境
        
        Args:
            ai_config: AI配置信息
            version_manager: 版本管理器实例
        """
        self.version_manager = version_manager
        self.ai_assistant = AIAssistant(
            config=ai_config, 
            tools=[self._create_requirement_analyzer_tool()]
        )

    def analyze_requirement(self, user_requirement: str) -> DecisionResult:
        """
        分析用户需求类型
        
        Args:
            user_requirement: 用户输入的需求
            
        Returns:
            DecisionResult: 决策结果
        """
        # 获取历史上下文
        history_context = self.version_manager.get_formatted_history()
        
        # 构建提示词
        prompt = f"""
# 任务
你需要分析用户的需求是否需要修改项目文件，如代码或文档，还是只需要回答问题。

# 历史上下文
{history_context}

# 当前用户需求
{user_requirement}

# 决策步骤
1. 仔细阅读用户当前的需求
2. 分析需求是否包含代码修改、文档修改、新增功能、修复bug、回滚代码等要求
3. 如果用户只是提问、咨询、请求解释或澄清，则判断为不需要修改代码
4、只要需要修改项目中的文件，则判断为需要修改代码

请使用requirement_analyzer工具返回决策结果。
"""

        # 发送给AI进行分析
        response = self.ai_assistant.generate_response(prompt, use_tools=True)
        
        # 如果没有得到有效决策，默认为需要修改代码
        if not isinstance(response, dict) or 'needs_code_modification' not in response:
            logger.warning("未获取到有效决策结果，默认为需要修改代码")
            return DecisionResult(
                needs_code_modification=True,
                reasoning="无法确定需求类型，默认为代码修改"
            )
            
        return DecisionResult(
            needs_code_modification=response['needs_code_modification'],
            reasoning=response['reasoning']
        )

    class _RequirementAnalyzerSchema(BaseModel):
        needs_code_modification: bool = Field(..., description="是否需要修改项目文件，如代码或文档")
        reasoning: Optional[str] = Field(None, description="决策理由")

    def _create_requirement_analyzer_tool(self) -> StructuredTool:
        """创建需求分析工具"""
        return StructuredTool.from_function(
            name="requirement_analyzer",
            description="分析用户需求是否需要修改项目文件，如代码或文档的工具",
            func=lambda **kwargs: kwargs,
            args_schema=self._RequirementAnalyzerSchema,
            return_direct=True
        )



import os
import re
from dataclasses import dataclass
from typing import List, Dict, Optional

from dotenv import load_dotenv
from jinja2 import Template

from core.ai import AIConfig
from core.file_memory import FileMemory
from core.file_selector import FileSelector


@dataclass
class PromptData:
    """提示数据类，封装生成提示所需的数据"""
    project_dir: str
    files: List[str]
    file_desc: Dict[str, str]
    requirement: str
    steps: Optional[str] = None


@dataclass
class ExtractedInfo:
    """提取的信息类，封装从提示中提取的数据"""
    project_dir: str
    files: List[str]
    file_desc: Dict[str, str]
    requirement: str
    steps: Optional[str] = None


class PromptGenerator:
    """
    生成结构化提示的工具类，用于根据文件描述、文件内容、历史执行信息和用户需求生成提示
    """
    
    # 提示模板
    PROMPT_TEMPLATE = """# 项目文件描述

{% for file_path in data.files %}
- {{ file_path }}: {{ data.file_desc.get(file_path, "无描述") }}
{% endfor %}

# 文件内容

{% for file_path in data.files %}
{% if file_exists(data.project_dir, file_path) %}
```
File: {{ file_path }}
{{ format_file_content(data.project_dir, file_path) }}
```

{% endif %}
{% endfor %}
{% if data.steps %}
# 历史执行信息

{{ data.steps }}

{% endif %}
# 用户需求

{{ data.requirement }}
"""

    @staticmethod
    def generatePrompt(data: PromptData) -> str:
        """
        根据提示数据生成结构化提示
        
        Args:
            data: 提示数据对象
            
        Returns:
            str: 生成的结构化提示
        """
        # 创建 Jinja2 模板
        template = Template(PromptGenerator.PROMPT_TEMPLATE)
        
        # 定义辅助函数
        def file_exists(project_dir, file_path):
            path = os.path.join(project_dir, file_path)
            return os.path.exists(path)
        
        def format_file_content(project_dir, file_path):
            try:
                path = os.path.join(project_dir, file_path)
                with open(path, "r", encoding="utf-8") as f:
                    content = f.read()
                
                # 添加行号
                lines = content.split("\n")
                numbered_content = ""
                for i, line in enumerate(lines, 1):
                    numbered_content += f"{i} {line}\n"
                
                return numbered_content.rstrip()
            except Exception as e:
                return f"无法读取文件内容: {str(e)}"
        
        # 渲染模板
        return template.render(
            data=data,
            file_exists=file_exists,
            format_file_content=format_file_content
        )

    @staticmethod
    def extractInfo(prompt: str) -> ExtractedInfo:
        """
        从提示中提取信息
        
        Args:
            prompt: 提示文本
            
        Returns:
            ExtractedInfo: 提取的信息对象
        """
        files = []
        file_desc = {}
        steps = ""
        requirement = ""
        
        # 提取文件描述
        desc_pattern = r"# 项目文件描述\n\n(.*?)(?=\n# )"
        desc_match = re.search(desc_pattern, prompt, re.DOTALL)
        if desc_match:
            desc_text = desc_match.group(1).strip()
            for line in desc_text.split("\n"):
                if line.startswith("- "):
                    parts = line[2:].split(": ", 1)
                    if len(parts) == 2:
                        file_path, desc = parts
                        files.append(file_path)
                        file_desc[file_path] = desc
        
        # 提取历史执行信息
        steps_pattern = r"# 历史执行信息\n\n(.*?)(?=\n# )"
        steps_match = re.search(steps_pattern, prompt, re.DOTALL)
        if steps_match:
            steps = steps_match.group(1).strip()
        
        # 提取用户需求
        req_pattern = r"# 用户需求\n\n(.*)"
        req_match = re.search(req_pattern, prompt, re.DOTALL)
        if req_match:
            requirement = req_match.group(1).strip()
        
        return ExtractedInfo(files, file_desc, steps, requirement)
    
    @staticmethod
    def formatFileContent(file_path: str) -> str:
        """
        格式化文件内容，添加行号
        
        Args:
            file_path: 文件路径
            
        Returns:
            str: 格式化后的文件内容
        """
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                content = f.read()
            
            # 添加行号
            lines = content.split("\n")
            numbered_content = ""
            for i, line in enumerate(lines, 1):
                numbered_content += f"{i} {line}\n"
            
            return numbered_content.rstrip()
        except Exception as e:
            return f"无法读取文件 {file_path}: {str(e)}"

if __name__ == "__main__":
    load_dotenv()
    requirement = """
    """
    project_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "../."))
    selector = FileSelector(project_dir, 2, ai_config=AIConfig(
        temperature=1,
        model_name="claude-3.7-sonnet"
    ))

    files = selector.select_files_for_requirement(requirement)

    descriptions = FileMemory.get_selected_file_descriptions("../.", files)

    data = PromptData(project_dir = "../.", files=files, file_desc=descriptions, requirement=requirement, steps=None)

    print(PromptGenerator.generatePrompt(data))




import os
from copy import copy
from dataclasses import dataclass, field

from dotenv import load_dotenv
from typing_extensions import Optional

from core.ai import AIAssistant, AIConfig
from core.diff import Diff
from core.log_manager import LogManager, LogConfig
from core.log_config import get_logger

logger = get_logger(__name__)

@dataclass
class CodeEngineerConfig:
    """代码工程师配置"""
    project_dir: str
    ai_config: AIConfig
    system_prompt: Optional[str] = None
    max_retries: int = 3

DEFAULT_PROMPT :str = '''
You will get instructions for code to write.
You will write a very long answer. Make sure that every detail of the architecture is, in the end, implemented as code.
Think step by step and reason yourself to the correct decisions to make sure we get it right.
Make changes to existing code and implement new code in the unified git diff syntax. When implementing new code, First lay out the names of the core classes, functions, methods that will be necessary, As well as a quick comment on their purpose.

You will output the content of each file necessary to achieve the goal, including ALL code.
Output requested code changes and new code in the unified "git diff" syntax. Example:

```diff
--- example.txt
+++ example.txt
@@ -6,3 +6,4 @@
     line content A
     line content B
+    new line added
-    original line X
+    modified line X with changes
@@ -26,4 +27,5 @@
         condition check:
-            action for condition A
+            if certain condition is met:
+                alternative action for condition A
         another condition check:
-            action for condition B
+            modified action for condition B
```

Example of a git diff creating a new file:

```diff
--- /dev/null
+++ new_file.txt
@@ -0,0 +1,3 @@
+First example line
+
+Last example line
```

RULES:
-A program will apply the diffs you generate exactly to the code, so diffs must be precise and unambiguous!
-Every diff must be fenced with triple backtick ```.
-The file names at the beginning of a diff, (lines starting with --- and +++) is the relative path to the file before and after the diff.
-LINES TO BE REMOVED (starting with single -) AND LINES TO BE RETAIN (no starting symbol) HAVE TO REPLICATE THE DIFFED HUNK OF THE CODE EXACTLY LINE BY LINE. KEEP THE NUMBER OF RETAIN LINES SMALL IF POSSIBLE.
-EACH LINE IN THE SOURCE FILES STARTS WITH A LINE NUMBER, WHICH IS NOT PART OF THE SOURCE CODE. NEVER TRANSFER THESE LINE NUMBERS TO THE DIFF HUNKS.
-AVOID STARTING A HUNK WITH AN EMPTY LINE.
-ENSURE ALL CHANGES ARE PROVIDED IN A SINGLE DIFF CHUNK PER FILE TO PREVENT MULTIPLE DIFFS ON THE SAME FILE.


As far as compatible with the user request, start with the "entrypoint" file, then go to the ones that are imported by that file, and so on.
Please note that the code should be fully functional. No placeholders.

Follow a language and framework appropriate best practice file naming convention.
Make sure that files contain all imports, types etc.  The code should be fully functional. Make sure that code in different files are compatible with each other.
Ensure to implement all code, if you are unsure, write a plausible implementation.
Include module dependency or package manager dependency definition file.
Before you finish, double check that all parts of the architecture is present in the files.

When you are done, write finish with "this concludes a fully working implementation".

Useful to know:
Almost always put different classes in different files.
Always use the programming language the user asks for.
For Python, you always create an appropriate requirements.txt file.
For NodeJS, you always create an appropriate package.json file.
Always add a comment briefly describing the purpose of the function definition.
Add comments explaining very complex bits of logic.
Always follow the best practices for the requested languages for folder/file structure and how to package the project.


Python toolbelt preferences:
- pytest
- dataclasses
'''

class CodeEngineer:
    """
    代码工程师类，负责处理用户的 prompt，与 AI 模型交互，解析 diff 并修改文件
    """

    def __init__(self, config: CodeEngineerConfig, log_manager: LogManager, diff: Diff):
        """
        初始化代码工程师

        Args:
            config: CodeEngineerConfig 实例，包含必要的配置信息
            log_manager: LogManager 实例，用于日志管理
        """
        self.config = config
        self.log_manager = log_manager
        self.diff = diff


        if config.system_prompt:
            self.system_prompt = config.system_prompt
        else:
            # 读取系统提示词
            self.system_prompt = self._read_system_prompt()

        self.ai_config = copy(config.ai_config)
        self.ai_config.sys_prompt = self.system_prompt
        self.ai_assistant = AIAssistant(config=self.ai_config)
        # 用于存储处理失败的文件
        self.failed_files = []
        # 用于存储修改的文件
        self.modified_files = []

    def _read_system_prompt(self) -> str:
        """
        读取系统提示词

        Returns:
            str: 系统提示词内容
        """
        try:
            system_prompt_path = os.path.join(self.config.project_dir, ".eng/system.txt")
            if os.path.exists(system_prompt_path):
                with open(system_prompt_path, "r", encoding="utf-8") as f:
                    return f.read()
            else:
                logger.warning(f"系统提示词文件不存在: {system_prompt_path}，使用默认提示词")
                return DEFAULT_PROMPT
        except Exception as e:
            logger.error(f"读取系统提示词失败: {str(e)}")
            return DEFAULT_PROMPT

    def process_prompt(self, prompt: str) ->  tuple[bool, Optional[str]]:
        """
        处理用户的 prompt，与 AI 模型交互，解析 diff 并修改文件

        Args:
            prompt: 用户的 prompt

        Returns:
            bool: 处理是否成功
            str: 模型返回结果
        """
        try:
            # 重置失败文件列表
            self.failed_files = []
            # 重置修改的文件列表
            self.diff_infos = []
            # 设置 AI 助手的系统提示词
            self.ai_assistant.config.sys_prompt = self.system_prompt
            
            # 调用 AI 模型生成响应
            response = self.ai_assistant.generate_response(prompt)
            
            # 解析响应中的 diff
            diffs = Diff.parse_diffs_from_text(response)
            
            if not diffs:
                logger.warning("未找到有效的 diff")
                return (False, None)
            
            # 处理每个 diff
            self.failed_files, self.diff_infos  = self.diff.process_diffs(diffs, self.config.project_dir)
            
            # 归档日志
            self.log_manager.archive_logs(
                sys_prompt=self.system_prompt,
                prompt=prompt,
                response=response,
                diff_infos=self.diff_infos
            )
            
            # 记录日志
            if self.modified_files:
                logger.info(f"本次修改了 {len(self.modified_files)} 个文件: " + 
                           ", ".join(self.modified_files[:5]) + 
                           ("..." if len(self.modified_files) > 5 else ""))
            
            # 如果有失败的文件，可以在这里处理
            if self.failed_files:
                logger.warning(f"有 {len(self.failed_files)} 个文件处理失败")
                # 这里可以添加失败文件的重试逻辑，但根据需求，暂时不实现
                return (False, response)
            
            return (True, response)
        except Exception as e:
            logger.error(f"处理 prompt 失败: {str(e)}")
            return (False, None)

    def retry_failed_files(self, ) -> bool:
        """
        重试处理失败的文件（钩子方法，暂不实现具体逻辑）

        Args:
            prompt: 用户的 prompt

        Returns:
            bool: 重试是否成功
        """
        # 这是一个钩子方法，用于未来扩展
        # 根据需求，暂时不实现具体逻辑

        return False

if __name__ == "__main__":
    load_dotenv()
    prompt = '''
    '''
    config = CodeEngineerConfig(project_dir="../.", ai_config=AIConfig(
        temperature=1,
        model_name="claude-3.7-sonnet"
    ))

    engineer = CodeEngineer(config, LogManager(LogConfig("../.", 1)), Diff(AIConfig(temperature=0.1,
                                                                                 model_name="gpt-4o")))
    engineer.process_prompt(prompt)
